/**************************************
This software was developed at the National Institute of Standards 
and Technology by employees of the Federal Government in the course 
of their official duties. Pursuant to title 17 Section 105 of the 
United States Code this software is not subject to copyright 
protection and is in the public domain. 
 
We would appreciate acknowledgement if the software is used.


Disclaimer:
-----------
There is no warranty, implied or otherwise, of any sort, nor any
guarantee of accuracy or correctness for any of the code in this
release.
**************************************/
/**************************************
File:btcolision.cpp
contains the modulators and random bit generators for GFSK and CCK
author: Amir Soltanian
Last update:  4/23/01
*************************************/

#include "802.h"

double qcoef[88] = // Gaussian filter coefficients for modulator. Ns=44 samples per bit 
{.0003384128078,     
 .0007483472705,     
 .001241740730,      
 .001831809714,      
 .002533043604,      
 .003361167254,      
 .004333069131,      
 .005466692340,      
 .006780887010,      
 .008295223730,      
 .01002976939 ,      
 .01200482814,       
 .01424065196,       
 .01675712682,       
 .01957344200,       
 .02270775110,       
 .02617683454,       
 .02999577350,       
 .03417764580,       
 .03873325341,       
 .04367089078,       
 .04899616180,       
 .05471185145,       
 .06081785680,       
 .06731117780,       
 .07418596970,       
 .08143365250,       
 .08904307290,       
 .09700071245,       
 .1052909327 ,       
 .1138962464 ,       
 .1227976077,        
 .1319747047,        
 .1414062494,        
 .1510702515,        
 .1609442679,        
 .1710056206,        
 .1812315788,        
 .1915994984,        
 .2020869202,        
 .2126716262,        
 .2233316570,        
 .2340452932,        
 .2447910090,        
 .2555474036,        
 .2662931193,        
 .2770067556,        
 .2876667864,        
 .2982514925,        
 .3087389142,        
 .3191068338,        
 .3293327920,        
 .3393941448,        
 .3492681611,        
 .3589321632,        
 .3683637078,        
 .3775408048,        
 .3864421662,        
 .3950474800,        
 .4033377002,        
 .4112953398,        
 .4189047602,        
 .4261524430,        
 .4330272350,        
 .4395205560,        
 .4456265612,        
 .4513422510,        
 .4566675220,        
 .4616051594,        
 .4661607670,        
 .4703426394,        
 .4741615782,        
 .4776306617,        
 .4807649708,        
 .4835812860,        
 .4860977608,        
 .4883335846,        
 .4903086434,        
 .4920431890,        
 .4935575258,        
 .4948717204,        
 .4960053436,       
 .4969772456,       
 .4978053692,       
 .4985066031,        
 .4990966721,        
 .4995900656,        
 .5000000000};        


/**************************************
/***** FUNCTION: generating random data for the Bluetooth interferenig signal
/***** INPUT: none
/***** OUTPUT: BTIntpacket[] global vector containing random data,
/*****************/
void  BTpacket(void){
	unsigned int newbit;
	for (int jj=0;jj<palen;jj++){ // A long PN sequence is generated by a shift register
		BTIntpacket[jj/16] <<= 1;
		newbit = ((iseed & IB20) >> 19) ^ ((iseed & IB3) >> 2) ;
		iseed = (iseed << 1 ) | newbit;
		BTIntpacket[jj/16] &= 0xfffe;
		BTIntpacket[jj/16] |= newbit; // pack every 16 bits in one word 
		}  
	for (jj=0;jj<palen;jj++){  // put each bit in seperate words
		BTIpacket[jj]= (BTIntpacket[jj/16] >> (jj%16) ) & 0x1;
		}
 	}

/**************************************
/***** FUNCTION: generating complex output of the Bluetooth interference modulator
/***** INPUT: df (carrier offset), bBT (interference amplitude), n (bit number) 
/*****        and  adjIpacket[] global variable containing random bits
/***** OUTPUT: adds BTIntInphase[] and BTIntQuad[] to Inphasecck[] and Quadcck[]
/*****************/
void BTInt(double df,double bBT, short int n){
short int i,j;
static short int a1; // initial bit
static double ph;  // initial phase
static double ph1;
double state [11*Nscck],tt[11*Nscck];
double BTIntQuad[11*Nscck],BTIntInphase[11*Nscck];
 
  
	if(!n){// for the first bit initialize the variables
		ph = rand()/(double)RAND_MAX; // random phase
		ph1=ph;                      // initial phase for GFSK
		ph = (1./hf) * rand()/(double)RAND_MAX; // random delay
		a1 = ALPHA_N_1;				// initial bit for GFSK	
		}

	for(i=0;i<8;i++){  //GFSK modulator for 8 consecutive Bluetooth bits
		for(j=0;j<44;j++){
			tt[j] = 2.*PI*(df* (j + i*44.+4.*n)/44.);
			if( tt[j]> (2*PI))
				tt[j] = fmod(tt[j],(2*PI));
			state[j]=(1.0-2.0*BTIpacket[i+n/11])*qcoef[j] + (1.0-2.0*a1)*qcoef[j+44]+ph;
 		  	BTIntInphase[j] = cos(tt[j]+2*PI*ph1 );
	 	 	BTIntQuad[j] = sin(tt[j]+2*PI*ph1 );
			BTIntInphase[j] *= bBT;  // change the amplitude of the interference according to CIR
			BTIntQuad[j] *= bBT;
			}
		// add the interference components
		for(j=0;j<11*Nscck;j++){
			Inphasecck[i*44+j] += BTIntInphase[j];
			Quadcck[i*44+j] += BTIntQuad[j];
			}//	for(j=0;j<88*Nscck;j++){
		if(!a1)
			ph += 0.5;
		else{
			ph -= 0.5;
			}
		a1 = BTIpacket[i+n/11];
		}//for(i=0;i<8;i++){
	

}

/**************************************
/***** FUNCTION: generating random data bits for the 802.11  signal
/***** INPUT: none
/***** OUTPUT: packet802[] global vector containing random data,
/*****************/
void  Packet802(void){
	unsigned int newbit;
	for (int jj=0;jj<palen;jj++){// A long PN sequence is generated by a shift register
		packet802[jj/16] <<= 1;
		newbit = ((iseed & IB20) >> 19) ^ ((iseed & IB3) >> 2) ;
		iseed = (iseed << 1 ) | newbit;
		packet802[jj/16] &= 0xfffe;
		packet802[jj/16] |= newbit;// pack every 16 bits in one word 
	    //packet802[jj/16] = 0x8010;
		}  
	for (jj=0;jj<palen;jj++){//put each bit in separate words
		Ipacket802[jj]= ((packet802[jj/16] >> (jj%16) ) & 0x1);
		}
}


/**************************************
/***** FUNCTION: generating complex output of the 802 CCK modulator
/***** INPUT: n (bit number) 
/*****        and  packet802[] global variable
/***** OUTPUT: Inphasecck[] and Quadcck[] global vectors
/*****************/
void  TxCCK256(short int n){
static double ph,I1B[RSLENGTH],Q1B[RSLENGTH];
double Istate[8*Nscck],Qstate[8*Nscck];
unsigned int Buf8;
static double ph1; 
double phase[4]={0, PI, PI/2, 3*PI/2};
double i1[8],q1[8];
int i,j,k,n1;

	// for the first bit initialize the buffer
	if(!n){
        ph =0;
		for( i=0;i<RSLENGTH;i++){// tx filter input
			I1B[i] = 0;
			Q1B[i] = 0.;
			}
		}
	for(i=0;i<8*Nscck;i++){  //tx filter input
			Istate[i]=0;
			Qstate[i]=0;
			}
	// in order to align cck and Bletooth pick 11 cck code words each containing
	//8 bits which has the same length of 8 Bluetooth bit.
	
	// 88*4 total samples= 11 cck code word= 88 (bits) * 4 (samples/bit) cck=
	// =8(bits)*44 (samples per bit) of Bluetooth.
 	
	for(k=0;k<11;k++){//11 code word
		n1=n+k*8;
		Buf8=(packet802[n1/16]>> (n1%16)) &0xff;// 8 bits bufffer
		// generate 8 output phase samples for the 8 bits
		for( i=0;i<8;i++){// 8 symbol per code word
				i1[i] = CInphase[Buf8][i];
				q1[i] =CQuad[Buf8][i];
		}

		// convolve with the tx pulse shaping filter
		// and generate 32 total samples per 8 bit code words
		for( j=0;j<8*Nscck;j+=Nscck){
			Istate[j]=i1[j/Nscck];
			Qstate[j]=q1[j/Nscck];
			}
		for( j=0;j<8*Nscck;j++){
		    Inphasecck[k*32+j] =0;
			Quadcck[k*32+j] =0;
			for(i=RSLENGTH-2;i>=0;i--){
				I1B[i+1] = I1B[i];
				Q1B[i+1] = Q1B[i];
				}
			I1B[0] = Istate[j];
			Q1B[0] = Qstate[j];
			for(i=0;i<RSLENGTH;i++){
				Inphasecck [k*32+j] += I1B[i] * hrsrc[i];
				Quadcck [k*32+j] += Q1B[i] * hrsrc[i];
				}//for(i=0;i<RSLENGTH;i++){
			}//for( j=0;j<8*Nscck;j++){
		}//for(k=0;k<11;k++){
}


