/**************************************
This software was developed at the National Institute of Standards 
and Technology by employees of the Federal Government in the course 
of their official duties. Pursuant to title 17 Section 105 of the 
United States Code this software is not subject to copyright 
protection and is in the public domain. 
 
We would appreciate acknowledgement if the software is used.

Disclaimer:
-----------
There is no warranty, implied or otherwise, of any sort, nor any
guarantee of accuracy or correctness for any of the code in this
release.

/**************************************
File:main.cpp
contains the main function the noise generator the receiver the filter tables
and the bit error rate calculator. 
author: Amir Soltanian
Last update:  4/23/01
*************************************/


#include "802.h"

long double cc;// callibration parameter
double EbNodb;// input carrier-to-noise ratio in dB

void main(void)
{
	int i,k,j;
	int npackets; // input number of packets
	double er=0,badj;
	double CAIRdb;  //input carrier-to-noise ratio in dB
    double ber[30][10];//output bit-error-rate 
	double ff; // input frequency offset in MHz
	FILE *fp;
   //****************************************************
   //   Generate Tables.                                
   //****************************************************
    RSRaisedCosine();// Pulse shaping filter for the CCK transmitter
	CodeTable();	// Generate all valid code word
    // **************************************************
   // Main loop over the number of packets
   // **************************************************
    
	for (int tt=0; tt <12; tt++){//different frequency offsets
		npackets= 50;
		printf("\n");
		for(k = 0;k <9; k ++) {
			ff = tt;			//MHz
			CAIRdb =-6+k;    //dB
			er=0.;
//			EbNodb = k ;                // different CNR values in dB
			EbNodb=30.0;                // high CNR interference-limited environment
			badj=sqrt(pow(10, -.1*CAIRdb)); // calculate interference amplitude
			n0=.5*Nscck*pow(10.,(-.1*EbNodb));// noise power
			cc=0;
			for(i = 0; i < npackets; i++) {
				Packet802();			// random bit 802.11 packet
				BTpacket();		        // Bluetooth interference packet	
				for(j = 0; j < palen; j+=88) {
					TxCCK256(j);		// CCK modulator
					BTInt(ff, badj, j); // Bluetooth GFSK modulator
					NoiseGen();			//AWGN noise	
					receiver3 (j);		// CCK receiver
					}//for(j = 0; j < palen; j++) {
				er += Error();          // bit error calculation
				} // end for i
//			cc /= (npackets*palen*Ns);	
			ber[k][tt] = er/(npackets * (palen-10) );//average BER
			printf (" CIR = %5.1f [db],   EbNo=%4.1f [db],   df=%4.0f [MHz]  BER= %4.2e\n", CAIRdb, EbNodb,ff, ber[k][tt]); 
			}//for(k = 0; i < 6; k++) {
    }//	for (int tt=0; tt <1; tt++){
    
	//Write results
	// This part have to be adjusted with each run

  	if( (fp=fopen("AWGN.doc","wb"))==NULL)
		 printf("can't open iqin.dat");
	fprintf(fp,"%% BER for CCK\n\n");
	//fprintf(fp,"%% df= 1MHZ, CNR=0,..29 [db]\n");
	for( tt=0;tt<12;tt++){
		fprintf(fp,"BER%2d=[",tt);
		fprintf(fp,"\n");
		for(int j=0;j<9;j++)
			fprintf(fp,"%4.2e ",ber[j][tt]);
		fprintf(fp,"];\n");
	}//  for( tt=0;tt<4;tt++){
 	fclose(fp);
	getch();
  }   //  end main gfsk1.C

//**************************************
/***** FUNCTION: generating AWGN noise using uniform random variables.
/***** INPUT: Global variable Inphase[] and Quad[]
/***** OUTPUT: NI and NQ components of the noise which are added to the 
               inphase (Inphasecck[]) and quadrature (Quadcck[]) components of the 
			   CCK modulator output.
/*****************/
 void  NoiseGen( void)
	{
	 double m1,m2;// uniform r.v.
	 double NI,NQ;
	for(int j=0; j<11*8*Nscck; j++)
			{
		 x1 = (171 * x1)%30269;
		 x2 = (172 * x2)%30307;
		 x3 = (170 * x3)%30323;
		 m1 = fmod( (double(x1)/30269. + double(x2)/30307. + double(x3)/30323.), 1.);
		
		 x4 = (171 * x4)%30269;
		 x5 = (172 * x5)%30307;
		 x6 = (170 * x6)%30323;
		 m2 = fmod( (double(x4)/30269. + double(x5)/30307. + double(x6)/30323.), 1.);
		 
		
		 NI=sqrt(-2*n0*log(m1))*cos(PI*2*m2);// Gaussian r.v.
		 NQ=sqrt(-2*n0*log(m1))*sin(PI*2*m2);// Gaussian r.v.
		 Inphasecck[j] += NI ;
		 Quadcck[j] += NQ ;
	
	}//	for(j=0; j<Ns*16*palen; j++)
 }


/**************************************
/***** FUNCTION: compare the input and output packet
/***** INPUT:  ppacket802[] and decod[] gobal vectors
/***** OUTPUT: er variable containing the total number of bit error in the entire packet
/*****************/

int Error( void ){
int j,k,er=0;
union {
      unsigned int d16;
      struct{
	    unsigned int d0:1;
	    }bit;
      }data1;

packet802[palen/16-1] &= 0xff;//ignore last eight bits
decod [palen/16-1] &= 0xff;
packet802[0] &= 0xfffc; // ignore first two bits
decod[0] &= 0xfffc;
for (j = 0; j < palen/16; j++){
	     data1.d16 = decod[j]^packet802[j];  //compare
		for (k = 0; k < 16; k++){
            er += data1.bit.d0;
			data1.d16>>=1;
			}//for (k = 0; k < 16; k++){
	     }//for (j = 0; j < palen; j++){

return (er);
}


/**************************************
/***** FUNCTION: generating all the possible code words
/***** INPUT: none
/***** OUTPUT: global vector CInphase[][] and CQuad[][]   
/*****************/

void CodeTable(void){
int i;
double phase[4]={0, PI, PI/2, 3*PI/2};
double ph1,ph2,ph3,ph4,cor[256]={0};
int d10,d32,d54,d76;

for(i=0;i<256;i++){
	d10 = i&0x3;	 //read d0 and d1
	d32 = (i>>2)&0x3;//read d2 and d3
	d54=  (i>>4)&0x3;//read d4 and d5
	d76 = (i>>6)&0x3;//read d6 and d7	
	 // QPSK phase
	ph1=phase[d10]; 
    ph2=phase[d32];
	ph3=phase[d54];
	ph4=phase[d76];
	//CCK phase output
	CInphase[i][0]=cos(ph1+ph2+ph3+ph4);
	CQuad[i][0]=sin(ph1+ph2+ph3+ph4);
	CInphase[i][1]=cos(ph1+ph3+ph4);
	CQuad[i][1]=sin(ph1+ph3+ph4);
	CInphase[i][2]=cos(ph1+ph2+ph4);
	CQuad[i][2]=sin(ph1+ph2+ph4);
	CInphase[i][3]=-cos(ph1+ph4);
	CQuad[i][3]=-sin(ph1+ph4);
	CInphase[i][4]=cos(ph1+ph2+ph3);
	CQuad[i][4]=sin(ph1+ph2+ph3);
	CInphase[i][5]=cos(ph1+ph3);
	CQuad[i][5]=sin(ph1+ph3);
	CInphase[i][6]=-cos(ph1+ph2);
	CQuad[i][6]=-sin(ph1+ph2);
	CInphase[i][7]=cos(ph1);
	CQuad[i][7]=sin(ph1);
	}
/*	measuring correlation
	for (i=0;i<256;i++){
	cor[i]=0;
	i1=q1=0;
	for(int j=0;j<8;j++){

	i1 +=(CInphase[0][j]*CInphase[i][j])+(CQuad[0][j]*CQuad[i][j]);
	q1 +=(CInphase[0][j]*CQuad[i][j])-(CQuad[0][j]*CInphase[i][j]);
		}
	cor[i]=i1*i1+q1*q1;
	}*/

}
/**************************************
/***** FUNCTION: generating Root-Sqared-Raised-Cosin filter coefficients
/***** INPUT: none
/***** OUTPUT: hraised[] global variable     
/*****************/
void RSRaisedCosine(void){
        double a=1.,hraised[RSLENGTH];
		for (int i = 0; i < RSLENGTH ; i++){
		double temp=   i - (RSLENGTH -1)/2.;
		 temp /= Ns;
		 temp *= 11;
        if((temp) && (1-16*temp*temp*a*a))
			hraised[i]= ( sin(PI*(1-a)*temp) + 4*a*temp*cos (PI*(1+a)*temp) )
			/( (PI*temp) * (1-16*temp*temp*a*a) );
		if(!temp)
			hraised[i] = 1-a+4*a/PI;
		if(!(1-16*temp*temp*a*a)){
			hraised[i]= (a/sqrt(2))*((1+2/PI)*sin(PI/(4*a))+(1-2/PI)*cos(PI/(4*a)));
			}
			}//	for (int i = 0; i < TDELAY; i++
		for ( i = 0; i < RSLENGTH ; i++)
			hrsrc[i] =hraised[i];
}


/**************************************
/***** FUNCTION: CCK receiver
/***** INPUT: Global variables Inphasecck[] and Quadcck[] and the bit number in the packet (n)
/***** OUTPUT: output bits in decod[] vector       
/*************************************/
void receiver3 (short int n){
		double X[11*8*Nscck] , Y[11*8*Nscck]; // Matched filter output
static	double IM[RSLENGTH], QM[RSLENGTH];    // buffer for matched filter convolution
		double ph1,ph2,ph3,ph4;
		double ph[4];
		double argi1,argq1,argi2,argq2,argi3,argq3,argi4,argq4;
unsigned short int d[4];
unsigned short int i, k;
int j,t;

	// Matched filter
	for (i = 0; i < 11*8*Nscck; i++){
		X[i] = 0.;
		Y[i] = 0.;
		for ( j = RSLENGTH -2; j >=0 ; j--){
			IM[j+1] = IM[j];
			QM[j+1] = QM[j];
			}//for (short int j=0;j<BPLENGTH;j++){
		IM[0] = Inphasecck[i]; 
		QM[0] = Quadcck[i];
		for ( j = 0; j < RSLENGTH; j++){
			X[i] += IM[j]*hrsrc[RSLENGTH-j-1];
			Y[i] += QM[j]*hrsrc[RSLENGTH-j-1];
			}//for (short int j=0;j<BPLENGTH;j++){
	}//for (short int i=0;i<totsamples-BPLENGTH;i++){

  // initialize the decod packet to zero
	if(!n){ 
		for (i = 0; i < palen/16 ; i++){
			decod[i] = 0;
			}//for ( int i=0;i<palen/16;i++){
		}//if(!n){

 // pick one sample per symbol
for (t = 0; t < 11; t++){
	for (i = 0; i < 8; i++){
		Inphasecck[i]=X[i*Nscck+t*32];
		Quadcck[i]=Y[i*Nscck+t*32];
		}
	Inphasecck[3]=-Inphasecck[3];
	Quadcck[3]=-Quadcck[3];
	Inphasecck[6]=-Inphasecck[6];
	Quadcck[6]=-Quadcck[6];

// calculate the received phases
	argi2=Inphasecck[0]*Inphasecck[1]+Quadcck[0]*Quadcck[1]+
		  Inphasecck[2]*Inphasecck[3]+Quadcck[2]*Quadcck[3]+
		  Inphasecck[4]*Inphasecck[5]+Quadcck[4]*Quadcck[5]+
	      Inphasecck[6]*Inphasecck[7]+Quadcck[6]*Quadcck[7];

	argq2=-Inphasecck[0]*Quadcck[1]+Quadcck[0]*Inphasecck[1]
		  -Inphasecck[2]*Quadcck[3]+Quadcck[2]*Inphasecck[3]
		  -Inphasecck[4]*Quadcck[5]+Quadcck[4]*Inphasecck[5]
	      -Inphasecck[6]*Quadcck[7]+Quadcck[6]*Inphasecck[7];

	argi3=Inphasecck[0]*Inphasecck[2]+Quadcck[0]*Quadcck[2]+
		  Inphasecck[1]*Inphasecck[3]+Quadcck[1]*Quadcck[3]+
		  Inphasecck[4]*Inphasecck[6]+Quadcck[4]*Quadcck[6]+
	      Inphasecck[5]*Inphasecck[7]+Quadcck[5]*Quadcck[7];

	argq3=-Inphasecck[0]*Quadcck[2]+Quadcck[0]*Inphasecck[2]
		  -Inphasecck[1]*Quadcck[3]+Quadcck[1]*Inphasecck[3]
		  -Inphasecck[4]*Quadcck[6]+Quadcck[4]*Inphasecck[6]
	      -Inphasecck[5]*Quadcck[7]+Quadcck[5]*Inphasecck[7];

	argi4=Inphasecck[0]*Inphasecck[4]+Quadcck[0]*Quadcck[4]+
		  Inphasecck[1]*Inphasecck[5]+Quadcck[1]*Quadcck[5]+
		  Inphasecck[2]*Inphasecck[6]+Quadcck[2]*Quadcck[6]+
	      Inphasecck[3]*Inphasecck[7]+Quadcck[3]*Quadcck[7];

	argq4=-Inphasecck[0]*Quadcck[4]+Quadcck[0]*Inphasecck[4]
		  -Inphasecck[1]*Quadcck[5]+Quadcck[1]*Inphasecck[5]
		  -Inphasecck[2]*Quadcck[6]+Quadcck[2]*Inphasecck[6]
	      -Inphasecck[3]*Quadcck[7]+Quadcck[3]*Inphasecck[7];

	ph2=atan2(argq2,argi2);
	ph3=atan2(argq3,argi3);
	ph4=atan2(argq4,argi4);

	argi1=Inphasecck[3]*cos(ph4)+Quadcck[3]*sin(ph4)+
		  Inphasecck[5]*cos(ph3)+Quadcck[5]*sin(ph3)+
		  Inphasecck[6]*cos(ph2)+Quadcck[6]*sin(ph2)+
		  Inphasecck[7];

	argq1=-Inphasecck[3]*sin(ph4)+Quadcck[3]*cos(ph4)
		  -Inphasecck[5]*sin(ph3)+Quadcck[5]*cos(ph3)
		  -Inphasecck[6]*sin(ph2)+Quadcck[6]*cos(ph2)
	      +Quadcck[7];
	ph1=atan2(argq1,argi1);
	ph[0]=ph1;
	ph[1]=ph2;
	ph[2]=ph3;
	ph[3]=ph4;
	//decod bits
	for ( i = 0; i <4; i++){
		if(ph[i]<0)
			ph[i]+=2*PI;
		if((0<=ph[i]) &&(ph[i]<PI/4))
			d[i]=0;
		if((7*PI/4<=ph[i]) &&(ph[i]<2*PI))
			d[i]=0;
		if((PI/4<=ph[i]) &&(ph[i]<3*PI/4))
			d[i]=2;
		if((3*PI/4<=ph[i]) &&(ph[i]<5*PI/4))
			d[i]=1;
		if((5*PI/4<=ph[i]) &&(ph[i]<7*PI/4))
			d[i]=3;
		}
	k=(d[3]<<6)|(d[2]<<4)|(d[1]<<2)|d[0];
	if((n)||(t)){
		k <<=((n+t*8-8)%16);
		decod[(n+t*8-8)/16] |= k;
		}
	}//	for (t = 0; t < 11; t++){
		
}